@@ -383,8 +383,8 @@ static int
 application_compare_by_name (const GAppInfo *app_a,
 			     const GAppInfo *app_b)
 {
-	return g_utf8_collate (g_app_info_get_name ((GAppInfo *)app_a),
-			       g_app_info_get_name ((GAppInfo *)app_b));
+	return g_utf8_collate (g_app_info_get_display_name ((GAppInfo *)app_a),
+			       g_app_info_get_display_name ((GAppInfo *)app_b));
 }
 
 static int
@@ -718,7 +718,7 @@ report_broken_symbolic_link (GtkWindow *parent_window, NautilusFile *file)
 	dialog = eel_show_yes_no_dialog (prompt, detail, _("Mo_ve to Trash"), GTK_STOCK_CANCEL,
 					 parent_window);
 
-	gtk_dialog_set_default_response (dialog, GTK_RESPONSE_YES);
+	gtk_dialog_set_default_response (dialog, GTK_RESPONSE_CANCEL);
 
 	/* Make this modal to avoid problems with reffing the view & file
 	 * to keep them around in case the view changes, which would then
@@ -1922,7 +1922,7 @@ activation_mount_not_mounted_callback (GObject *source_object,
 		     error->code != G_IO_ERROR_FAILED_HANDLED &&
 		     error->code != G_IO_ERROR_ALREADY_MOUNTED)) {
 			eel_show_error_dialog (_("Unable to mount location"),
-					       error->message, NULL);
+					       error->message, parameters->parent_window);
 		}
 
 		if (error->domain != G_IO_ERROR ||
@@ -2195,7 +2195,7 @@ activation_mountable_mounted (NautilusFile  *file,
 		     error->code != G_IO_ERROR_FAILED_HANDLED &&
 		     error->code != G_IO_ERROR_ALREADY_MOUNTED)) {
 			eel_show_error_dialog (_("Unable to mount location"),
-					       error->message, NULL);
+					       error->message, parameters->parent_window);
 		}
 
 		if (error->code == G_IO_ERROR_CANCELLED) {
