@@ -29,6 +29,7 @@
 #include "nautilus-open-with-dialog.h"
 #include "nautilus-signaller.h"
 
+#include <eel/eel-glib-extensions.h>
 #include <eel/eel-stock-dialogs.h>
 
 #include <string.h>
@@ -44,11 +45,11 @@ struct _NautilusOpenWithDialogDetails {
 	
 	char *content_type;
 	char *extension;
-	char *type_description;
 
 	GtkWidget *label;
 	GtkWidget *entry;
 	GtkWidget *button;
+	GtkWidget *checkbox;
 
 	GtkWidget *desc_label;
 
@@ -59,6 +60,8 @@ struct _NautilusOpenWithDialogDetails {
 	GSList	      *add_icon_paths;
 	gint	       add_items_idle_id;
 	gint	       add_icons_idle_id;
+
+	gboolean add_mode;
 };
 
 enum {
@@ -81,8 +84,8 @@ enum {
 	LAST_SIGNAL
 };
 
-static gpointer parent_class;
 static guint signals[LAST_SIGNAL] = { 0 };
+G_DEFINE_TYPE (NautilusOpenWithDialog, nautilus_open_with_dialog, GTK_TYPE_DIALOG); 
 
 static void
 nautilus_open_with_dialog_finalize (GObject *object)
@@ -104,17 +107,16 @@ nautilus_open_with_dialog_finalize (GObject *object)
 	}
 	g_free (dialog->details->content_type);
 	g_free (dialog->details->extension);
-	g_free (dialog->details->type_description);
 
 	g_free (dialog->details);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (nautilus_open_with_dialog_parent_class)->finalize (object);
 }
 
 static void
 nautilus_open_with_dialog_destroy (GtkObject *object)
 {
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	GTK_OBJECT_CLASS (nautilus_open_with_dialog_parent_class)->destroy (object);
 }
 
 /* An application is valid if:
@@ -212,8 +214,9 @@ add_or_find_application (NautilusOpenWithDialog *dialog)
 	char *app_name;
 	const char *commandline;
 	GError *error;
-	gboolean success;
+	gboolean success, should_set_default;
 	char *message;
+	GList *applications;
 
 	error = NULL;
 	app = NULL;
@@ -241,18 +244,36 @@ add_or_find_application (NautilusOpenWithDialog *dialog)
 		return NULL;
 	}
 
+	should_set_default = (dialog->details->add_mode) ||
+		(!dialog->details->add_mode &&
+		 gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->details->checkbox)));
+	success = TRUE;
 
-	if (dialog->details->content_type) {
-		success = g_app_info_add_supports_type (app,
-							dialog->details->content_type,
-							&error);
+	if (should_set_default) {
+		if (dialog->details->content_type) {
+			success = g_app_info_set_as_default_for_type (app,
+								      dialog->details->content_type,
+								      &error);
+		} else {
+			success = g_app_info_set_as_default_for_extension (app,
+									   dialog->details->extension,
+									   &error);
+		}
 	} else {
-		success = g_app_info_set_as_default_for_extension (app,
-								   dialog->details->extension,
-								   &error);
+		applications = g_app_info_get_all_for_type (dialog->details->content_type);
+		if (dialog->details->content_type && applications != NULL) {
+			/* we don't care about reporting errors here */
+			g_app_info_add_supports_type (app,
+						      dialog->details->content_type,
+						      NULL);
+		}
+
+		if (applications != NULL) {
+			eel_g_object_list_free (applications);
+		}
 	}
-	
-	if (!success) {
+
+	if (!success && should_set_default) {
 		message = g_strdup_printf (_("Could not set application as the default: %s"), error->message);
 		eel_show_error_dialog (_("Could not set as default application"),
 				       message,
@@ -263,7 +284,6 @@ add_or_find_application (NautilusOpenWithDialog *dialog)
 
 	g_signal_emit_by_name (nautilus_signaller_get_current (),
 			       "mime_data_changed");
-
 	return app;
 }
 
@@ -341,8 +361,6 @@ nautilus_open_with_dialog_class_init (NautilusOpenWithDialogClass *class)
 	GObjectClass *gobject_class;
 	GtkObjectClass *object_class;
 
-	parent_class = g_type_class_peek_parent (class);
-
 	gobject_class = G_OBJECT_CLASS (class);
 	gobject_class->finalize = nautilus_open_with_dialog_finalize;
 
@@ -382,7 +400,7 @@ chooser_response_cb (GtkFileChooser *chooser,
 
 			gtk_entry_set_text (GTK_ENTRY (dialog->details->entry),
 					    quoted_text);
-			gtk_entry_set_position (GTK_ENTRY (dialog->details->entry), -1);
+			gtk_editable_set_position (GTK_EDITABLE (dialog->details->entry), -1);
 			g_free (quoted_text);
 			g_free (filename);
 		}
@@ -598,9 +616,9 @@ nautilus_open_with_dialog_add_items_idle (NautilusOpenWithDialog *dialog)
 {
 	GtkCellRenderer   *renderer;
 	GtkTreeViewColumn *column;
-	GList            *all_applications;
-	GList            *l;
-	const char        *prev_name;
+	GtkTreeModel      *sort;
+	GList             *all_applications;
+	GList             *l;
 
 	/* create list store */
 	dialog->details->program_list_store = gtk_list_store_new (NUM_COLUMNS,
@@ -610,10 +628,9 @@ nautilus_open_with_dialog_add_items_idle (NautilusOpenWithDialog *dialog)
 								  G_TYPE_STRING,
 								  G_TYPE_STRING,
 								  G_TYPE_STRING);
-
+	sort = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (dialog->details->program_list_store));
 	all_applications = g_app_info_get_all ();
 	
-	prev_name = NULL;
 	for (l = all_applications; l; l = l->next) {
 		GAppInfo *app = l->data;
 		GtkTreeIter     iter;
@@ -628,7 +645,7 @@ nautilus_open_with_dialog_add_items_idle (NautilusOpenWithDialog *dialog)
 				    COLUMN_APP_INFO,  app,
 				    COLUMN_ICON,      NULL,
 				    COLUMN_GICON,     g_app_info_get_icon (app),
-				    COLUMN_NAME,      g_app_info_get_name (app),
+				    COLUMN_NAME,      g_app_info_get_display_name (app),
 				    COLUMN_COMMENT,   g_app_info_get_description (app),
 				    COLUMN_EXEC,      g_app_info_get_executable,
 				    -1);
@@ -641,7 +658,9 @@ nautilus_open_with_dialog_add_items_idle (NautilusOpenWithDialog *dialog)
 	g_list_free (all_applications);
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (dialog->details->program_list), 
-				 GTK_TREE_MODEL (dialog->details->program_list_store));
+				 GTK_TREE_MODEL (sort));
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (sort),
+					      COLUMN_NAME, GTK_SORT_ASCENDING);
 	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (dialog->details->program_list),
 					     nautilus_open_with_search_equal_func,
 					     NULL, NULL);
@@ -658,7 +677,7 @@ nautilus_open_with_dialog_add_items_idle (NautilusOpenWithDialog *dialog)
 	gtk_tree_view_column_set_attributes (column, renderer,
                                              "text", COLUMN_NAME,
                                              NULL);
-					          
+	gtk_tree_view_column_set_sort_column_id (column, COLUMN_NAME);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (dialog->details->program_list), column);
 
 	dialog->details->add_icon_paths = g_slist_reverse (dialog->details->add_icon_paths);
@@ -745,7 +764,7 @@ expander_toggled (GtkWidget *expander, NautilusOpenWithDialog *dialog)
 }
 
 static void
-nautilus_open_with_dialog_instance_init (NautilusOpenWithDialog *dialog)
+nautilus_open_with_dialog_init (NautilusOpenWithDialog *dialog)
 {
 	GtkWidget *hbox;
 	GtkWidget *vbox;
@@ -843,6 +862,13 @@ nautilus_open_with_dialog_instance_init (NautilusOpenWithDialog *dialog)
 	gtk_box_pack_start (GTK_BOX (hbox), dialog->details->button, FALSE, FALSE, 0);
 	gtk_widget_show (dialog->details->button);
 
+	/* Add remember this application checkbox - only visible in open mode */
+	dialog->details->checkbox = gtk_check_button_new ();
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->details->checkbox), TRUE);
+	gtk_button_set_use_underline (GTK_BUTTON (dialog->details->checkbox), TRUE);
+	gtk_widget_show (GTK_WIDGET (dialog->details->checkbox));
+	gtk_box_pack_start (GTK_BOX (vbox), dialog->details->checkbox, FALSE, FALSE, 0);
+
         gtk_dialog_add_button (GTK_DIALOG (dialog),
                                GTK_STOCK_REMOVE,
 			       RESPONSE_REMOVE);
@@ -913,12 +939,14 @@ static void
 set_uri_and_type (NautilusOpenWithDialog *dialog,
 		  const char *uri,
 		  const char *mime_type,
-		  const char *passed_extension)
+		  const char *passed_extension,
+		  gboolean add_mode)
 {
 	char *label;
 	char *emname;
 	char *name, *extension;
 	char *description;
+	char *checkbox_text;
 	
 	name = NULL;
 	extension = NULL;
@@ -939,9 +967,29 @@ set_uri_and_type (NautilusOpenWithDialog *dialog,
 	if (extension != NULL &&
 	    g_content_type_is_unknown (mime_type)) {
 		dialog->details->extension = g_strdup (extension);
-		/* the %s here is a file extension */
-		dialog->details->type_description =
-			g_strdup_printf (_("%s document"), extension);
+
+		if (name != NULL) {
+			emname = g_strdup_printf ("<i>%s</i>", name);
+			if (add_mode) {
+				/* first %s is a filename and second %s is a file extension */
+				label = g_strdup_printf (_("Open %s and other %s document with:"),
+							 emname, dialog->details->extension);
+			} else {
+				/* the %s here is a file name */
+				label = g_strdup_printf (_("Open %s with:"), emname);
+				checkbox_text = g_strdup_printf (_("_Remember this application for %s documents"), 
+								 dialog->details->extension);
+
+				gtk_button_set_label (GTK_BUTTON (dialog->details->checkbox), checkbox_text);
+				g_free (checkbox_text);
+			}
+			g_free (emname);
+		} else {
+			/* Only in add mode - the %s here is a file extension */
+			label = g_strdup_printf (_("Open all %s documents with:"),
+						 dialog->details->extension);
+		}
+		g_free (extension);
 	} else {
 		dialog->details->content_type = g_strdup (mime_type);
 		description = g_content_type_get_description (mime_type);
@@ -950,17 +998,39 @@ set_uri_and_type (NautilusOpenWithDialog *dialog,
 			description = g_strdup (_("Unknown"));
 		}
 
-		dialog->details->type_description = description;
-	}
-	g_free (extension);
+		if (name != NULL) {
+			emname = g_strdup_printf ("<i>%s</i>", name);
+			if (add_mode) {
+				/* First %s is a filename, second is a description
+				 * of the type, eg "plain text document" */
+				label = g_strdup_printf (_("Open %s and other \"%s\" files with:"), 
+							 emname, description);
+			} else {
+				/* %s is a filename */
+				label = g_strdup_printf (_("Open %s with:"), emname);
+				/* %s is a file type description */
+				checkbox_text = g_strdup_printf (_("_Remember this application for \"%s\" files"),
+								 description);
 
-	if (name != NULL) {
-		emname = g_strdup_printf ("<i>%s</i>", name);
-		label = g_strdup_printf (_("Open %s and other files of type \"%s\" with:"), emname, dialog->details->type_description);
-		g_free (emname);
-	} else {
-		label = g_strdup_printf (_("Open all files of type \"%s\" with:"),
-					 dialog->details->type_description);
+				gtk_button_set_label (GTK_BUTTON (dialog->details->checkbox), checkbox_text);
+				g_free (checkbox_text);
+			}
+			g_free (emname);
+		} else {
+			/* Only in add mode */
+			label = g_strdup_printf (_("Open all \"%s\" files with:"), description);
+		}
+
+		g_free (description);
+	}
+
+	dialog->details->add_mode = add_mode;
+	if (add_mode) {
+		gtk_widget_hide (dialog->details->checkbox);
+
+		gtk_label_set_text_with_mnemonic (GTK_LABEL (dialog->details->open_label),
+						  _("_Add"));
+		gtk_window_set_title (GTK_WINDOW (dialog), _("Add Application"));
 	}
 
 	gtk_label_set_markup (GTK_LABEL (dialog->details->label), label);
@@ -969,72 +1039,49 @@ set_uri_and_type (NautilusOpenWithDialog *dialog,
 	g_free (name);
 }
 
-GtkWidget *
-nautilus_open_with_dialog_new (const char *uri,
-			       const char *mime_type,
-			       const char *extension)
+
+static GtkWidget *
+real_nautilus_open_with_dialog_new (const char *uri,
+				    const char *mime_type,
+				    const char *extension,
+				    gboolean add_mode)
 {
 	GtkWidget *dialog;
 
 	dialog = gtk_widget_new (NAUTILUS_TYPE_OPEN_WITH_DIALOG, NULL);
 
-	set_uri_and_type (NAUTILUS_OPEN_WITH_DIALOG (dialog), uri, mime_type, extension);
+	set_uri_and_type (NAUTILUS_OPEN_WITH_DIALOG (dialog), uri, mime_type, extension, add_mode);
 
 	return dialog;
 }
 
-GtkWidget* 
+GtkWidget *
+nautilus_open_with_dialog_new (const char *uri,
+			       const char *mime_type,
+			       const char *extension)
+{
+	return real_nautilus_open_with_dialog_new (uri, mime_type, extension, FALSE);
+}
+
+GtkWidget * 
 nautilus_add_application_dialog_new (const char *uri,
-				const char *mime_type)
+				     const char *mime_type)
 {
 	NautilusOpenWithDialog *dialog;
 	
-	dialog = NAUTILUS_OPEN_WITH_DIALOG (nautilus_open_with_dialog_new (uri, mime_type, NULL));
-	
-	gtk_label_set_text_with_mnemonic (GTK_LABEL (dialog->details->open_label),
-					  _("_Add"));
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Add Application"));
+	dialog = NAUTILUS_OPEN_WITH_DIALOG (real_nautilus_open_with_dialog_new (uri, mime_type, NULL, TRUE));
 
 	return GTK_WIDGET (dialog);
 }
 
-GtkWidget* 
+GtkWidget * 
 nautilus_add_application_dialog_new_for_multiple_files (const char *extension,
 							const char *mime_type)
 {
 	NautilusOpenWithDialog *dialog;
 	
-	dialog = NAUTILUS_OPEN_WITH_DIALOG (nautilus_open_with_dialog_new (NULL, mime_type, extension));
+	dialog = NAUTILUS_OPEN_WITH_DIALOG (real_nautilus_open_with_dialog_new (NULL, mime_type, extension, TRUE));
 	
-	gtk_label_set_text_with_mnemonic (GTK_LABEL (dialog->details->open_label),
-					  _("_Add"));
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Add Application"));
-
 	return GTK_WIDGET (dialog);
 }
 
-GType
-nautilus_open_with_dialog_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		const GTypeInfo info = {
-			sizeof (NautilusOpenWithDialogClass),
-			NULL,
-			NULL,
-			(GClassInitFunc)nautilus_open_with_dialog_class_init,
-			NULL,
-			NULL,
-			sizeof (NautilusOpenWithDialog),
-			0,
-			(GInstanceInitFunc)nautilus_open_with_dialog_instance_init,
-		};
-
-		type = g_type_register_static (GTK_TYPE_DIALOG,
-					       "NautilusOpenWithDialog",
-					       &info, 0);
-	}
-
-	return type;
-}
